@@ -65,7 +65,8 @@ mod array_impl;
 
 pub mod messageitem;
 
-pub use self::msgarg::{Arg, FixedArray, Get, DictKey, Append, RefArg, AppendAll, ReadAll, ArgAll, cast, cast_mut};
+pub use self::msgarg::{Arg, FixedArray, Get, DictKey, Append, RefArg, AppendAll, ReadAll, ArgAll,
+    cast, cast_mut, prop_cast, PropMap};
 pub use self::array_impl::{Array, Dict};
 pub use self::variantstruct_impl::Variant;
 
@@ -73,7 +74,9 @@ use std::{fmt, mem, ptr, error};
 use crate::{ffi, Message, Signature, Path};
 use std::ffi::{CStr, CString};
 use std::os::raw::{c_void, c_int};
+#[cfg(unix)]
 use std::os::unix::io::{RawFd, AsRawFd, FromRawFd, IntoRawFd};
+use std::collections::VecDeque;
 
 fn check(f: &str, i: u32) { if i == 0 { panic!("D-Bus error: '{}' failed", f) }}
 
@@ -86,9 +89,11 @@ fn ffi_iter() -> ffi::DBusMessageIter {
 /// when the scope ends.
 #[derive(Debug, PartialEq, PartialOrd)]
 pub struct OwnedFd {
+    #[cfg(unix)]
     fd: RawFd
 }
 
+#[cfg(unix)]
 impl OwnedFd {
     /// Create a new OwnedFd from a RawFd.
     ///
@@ -106,6 +111,7 @@ impl OwnedFd {
     }
 }
 
+#[cfg(unix)]
 impl Drop for OwnedFd {
     fn drop(&mut self) {
         unsafe { libc::close(self.fd); }
@@ -113,25 +119,34 @@ impl Drop for OwnedFd {
 }
 
 impl Clone for OwnedFd {
+    #[cfg(unix)]
     fn clone(&self) -> OwnedFd {
         let x = unsafe { libc::dup(self.fd) };
         if x == -1 { panic!("Duplicating file descriptor failed") }
         unsafe { OwnedFd::new(x) }
     }
+
+    #[cfg(windows)]
+    fn clone(&self) -> OwnedFd {
+        OwnedFd {}
+    }
 }
 
+#[cfg(unix)]
 impl AsRawFd for OwnedFd {
     fn as_raw_fd(&self) -> RawFd {
         self.fd
     }
 }
 
+#[cfg(unix)]
 impl IntoRawFd for OwnedFd {
     fn into_raw_fd(self) -> RawFd {
         self.into_fd()
     }
 }
 
+#[cfg(unix)]
 impl FromRawFd for OwnedFd {
     unsafe fn from_raw_fd(fd: RawFd) -> Self { OwnedFd::new(fd) }
 }
@@ -260,8 +275,8 @@ impl<'a> Iter<'a> {
             ArgType::Int64 => Box::new(self.get::<i64>().unwrap()),
             ArgType::UInt64 => Box::new(self.get::<u64>().unwrap()),
             ArgType::Double => Box::new(self.get::<f64>().unwrap()),
-            ArgType::UnixFd => Box::new(self.get::<OwnedFd>().unwrap()),
-            ArgType::Struct => Box::new(self.recurse(ArgType::Struct).unwrap().collect::<Vec<_>>()),
+            ArgType::UnixFd => Box::new(self.get::<std::fs::File>().unwrap()),
+            ArgType::Struct => Box::new(self.recurse(ArgType::Struct).unwrap().collect::<VecDeque<_>>()),
             ArgType::ObjectPath => Box::new(self.get::<Path>().unwrap().into_static()),
             ArgType::Signature => Box::new(self.get::<Signature>().unwrap().into_static()),
         })
@@ -273,7 +288,7 @@ impl<'a> Iter<'a> {
             let c = ffi::dbus_message_iter_get_signature(&mut self.0);
             assert!(c != ptr::null_mut());
             let cc = CStr::from_ptr(c);
-            let r = Signature::new(cc.to_bytes());
+            let r = Signature::new(std::str::from_utf8(cc.to_bytes()).unwrap());
             ffi::dbus_free(c as *mut c_void);
             r.unwrap()
         }
@@ -402,7 +417,7 @@ pub enum ArgType {
     UInt64 = ffi::DBUS_TYPE_UINT64 as u8,
     /// f64
     Double = ffi::DBUS_TYPE_DOUBLE as u8,
-    /// OwnedFd
+    /// File
     UnixFd = ffi::DBUS_TYPE_UNIX_FD as u8,
     /// Use tuples or Vec<Box<dyn RefArg>> to read/write structs.
     Struct = ffi::DBUS_TYPE_STRUCT as u8,
@@ -420,7 +435,7 @@ const ALL_ARG_TYPES: [(ArgType, &str); 18] =
     (ArgType::DictEntry, "Dict entry"),
     (ArgType::ObjectPath, "Path"),
     (ArgType::Signature, "Signature"),
-    (ArgType::UnixFd, "OwnedFd"),
+    (ArgType::UnixFd, "File"),
     (ArgType::Boolean, "bool"),
     (ArgType::Byte, "u8"),
     (ArgType::Int16, "i16"),
